/*
 *              metadata.c
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <stdint.h>
#include  <unistd.h>
#include  "string.h"

#include  "../floatimg.h"

extern int verbosity;           /* must be declared around main() */

/* ---------------------------------------------------------------- */
static void puts_magic_8(char *ptr, FILE *fp)
{
int		foo;

fputs("magic                = ", fp);
fputc('[', fp);
for (foo=0; foo<8; foo++) {
	fputc(ptr[foo], fp);
	}
fputc(']', fp); fputc('\n', fp);
fflush(fp);
}
/* ---------------------------------------------------------------- */
int fimg_show_metadata(FimgMetaData *pmd, char *title, int notused)
{
int		foo;

fprintf(stderr, ">>> %s ( %p '%s' 0x%08x )\n", __func__,
				pmd, title, notused);

if (verbosity) {
	fprintf(stderr, "sizeof(metadata)     = %ld\n",    \
				sizeof(FimgMetaData));
	puts_magic_8(pmd->magic, stderr);
	}

/* SHOW TIMESTAMP HERE */
fprintf(stderr, "creator pid          = %ld\n", pmd->pid);
fprintf(stderr, "counter              = %d\n", pmd->count);
fprintf(stderr, "float value          = %.3f\n", pmd->fval);
fprintf(stderr, "id camera            = '%s'\n", pmd->idcam);
fprintf(stderr, "origin               = 0x%x\n", pmd->origin);

fputs("reserved words are:\n      ", stderr);
for (foo=0; foo<8; foo++) {
	fprintf(stderr, "   0x%08x", pmd->reserved[foo]);
	if (3 == foo) fputs("\n      ", stderr);
	}
fputc('\n', stderr);
return 0;
}
/* ---------------------------------------------------------------- */
int fimg_default_metadata(FimgMetaData *pmd)
{

memset(pmd, 0, sizeof(FimgMetaData));
memcpy(pmd->magic, "metadata", 8);
/* set timestamp here ? */
pmd->pid = getpid();
pmd->count    = 0;
pmd->fval     = 255.0;
strcpy(pmd->idcam, "<unknow>");
pmd->origin   = 0x55555555;

return 0;
}
/* ---------------------------------------------------------------- */
int fimg_get_metadata_from_file(char *fname, FimgMetaData *pmd)
{
FILE			*fp;
FimgFileHead		filehead;
FimgMetaData		metadata;
int			foo;

// #if DEBUG_LEVEL
fprintf(stderr, ">>> %s ( '%s' %p )\n", __func__, fname, pmd);
// #endif

if (NULL==(fp=fopen(fname, "r"))) {
	perror(fname);
	return -1;
	}

foo = fread(&filehead, sizeof(FimgFileHead), 1, fp);
if (1 != foo) {
	fprintf(stderr, "short read (%d) on %s (head)\n", foo, fname);
	fclose(fp);
	return -2;
	}

if (memcmp(filehead.magic, "FIMG", 4)) {
	fprintf(stderr, "'%s' is not a fimg file.\n", fname);
	fclose(fp);
	return -3;
	}

if ('a' != filehead.magic[4]) {
	fprintf(stderr, "file '%s' have no metadata.\n", fname);
	fclose(fp);
	return -4;
	}

foo = fread(&metadata, sizeof(FimgMetaData), 1, fp);
if (1 != foo) {
	fprintf(stderr, "short read on %s (metadata)\n", fname);
	fclose(fp);
	return -5;
	}

fclose(fp);		/* got all needed datas */

if (memcmp(metadata.magic, "metadata", 8)) {
	fprintf(stderr, "'%s' invalid metadata.\n", fname);
	return -6;
	}

memcpy(pmd, &metadata, sizeof(FimgMetaData));

return 0;
}
/* ---------------------------------------------------------------- */
